from aiosmb.protocol.smb.commands.negotiate_req import SMB_COM_NEGOTIATE_REQ

#add new commands as they are implemented.
#do not forget to add the same commands to command_cdoes as well!


__all__ = [
	'SMB_COM_NEGOTIATE_REQ',

]